package net.wanji.common.utils;


import lombok.extern.slf4j.Slf4j;
import net.wanji.common.utils.tool.StringUtils;
import okhttp3.*;

import java.io.IOException;

/**
 * @ClassName OkHttpClient
 * @Description OkHttpClient
 * @Author zhouleilei
 * @Date 2024/11/3 18:11
 */
@Slf4j
public class OkHttpClientUtil {

    public static String xmlPost(String url, String xmlContent) {
        if (StringUtils.isBlank(url) || StringUtils.isBlank(xmlContent)) {
            return null;
        }
        // 创建 OkHttpClient 实例
        OkHttpClient client = new OkHttpClient();
        // 创建请求体
        RequestBody body = RequestBody.create(xmlContent, MediaType.get("application/xml; charset=utf-8"));
        // 创建请求
        Request request = new Request.Builder()
                .url(url)
                .post(body)
                .build();
        // 执行请求
        try (Response response = client.newCall(request).execute()) {
            // 获取响应体
            if (response.code() == 200 && response.body() != null) {
                String responseString = response.body().string();
                return responseString;
            }
        } catch (IOException e) {
            e.printStackTrace();
        }
        return null;
    }


    public static String jsonPost(String url, String json) throws Exception {
        if (StringUtils.isBlank(url) || StringUtils.isBlank(json)) {
            return null;
        }
        // 创建 OkHttpClient 实例
        OkHttpClient client = new OkHttpClient();
        // 创建请求体
        RequestBody body = RequestBody.create(json, MediaType.get("application/json; charset=utf-8"));
        // 创建请求
        Request request = new Request.Builder()
                .url(url)
                .post(body)
                .build();
        // 执行请求
        try (Response response = client.newCall(request).execute()) {
            // 获取响应体
            if (response.code() == 200 && response.body() != null) {
                String responseString = response.body().string();
                log.info("远程服务调用成功，url：{}", url);
                return responseString;
            }
        } catch (Exception e) {
            log.error("OkHttpClientUtil远程服务url:{}, 调用异常：{}", url, e.getMessage());
            throw new Exception();
        }
        return null;
    }

    public static String get(String url) throws Exception {
        if (StringUtils.isBlank(url)) {
            return null;
        }
        // 创建 OkHttpClient 客户端
        OkHttpClient client = new OkHttpClient();

        // 构建请求
        Request request = new Request.Builder()
                .url(url)
                .get()
                .build();

        // 执行请求
        try (Response response = client.newCall(request).execute()) {
            // 获取响应体
            if (response.isSuccessful()) {
                String responseString = response.body().string();
                log.info("远程服务调用成功，url：{}", url);
                return responseString;
            }
        } catch (Exception e) {
            log.error("OkHttpClientUtil远程服务url:{}, 调用异常：{}", url, e.getMessage());
            throw new Exception();
        }
        return null;
    }
}
